SystemOrganization addCategory: #Bundleview!

Object subclass: #BSpline
	instanceVariableNames: 'tesselatedPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bundleview'!

!BSpline methodsFor: 'accessing' stamp: 'CR 10/21/2012 17:31'!
addTesselatedPoint: aPoint

	tesselatedPoints add: aPoint.! !

!BSpline methodsFor: 'rendering' stamp: 'CR 10/23/2012 14:24'!
calculateBSpline: aPointList
	
	| skm1 cpim1 cpi cpip1 sk line rkm1 lk |
	(aPointList size < 4) ifTrue: [^self].
	self addTesselatedPoint: (aPointList at: 1).
	
	skm1 := aPointList at: 1.
	cpim1 := skm1.
	cpi := aPointList at: 2.
	
	2 to: (aPointList size - 2) do: [ :i | | pointsForCurve |
		pointsForCurve := OrderedCollection new.
		(i = (aPointList size - 2))
			ifTrue: [sk := cpi.]
			ifFalse: [cpip1 := aPointList at: (i + 1).
				sk := ((cpim1 + (4 * cpi) + cpip1)/6).].
		line := cpi - cpim1.
		rkm1 := cpim1 + ((line * 1)/3).
		lk := cpim1 + ((line *2)/3).
		pointsForCurve add: skm1.
		pointsForCurve add: rkm1.
		pointsForCurve add: lk.
		pointsForCurve add: sk.
		self calculateBezier: pointsForCurve.
		skm1 := sk.
		cpim1 := cpi.
		cpi := cpip1.].! !

!BSpline methodsFor: 'rendering' stamp: 'CR 10/23/2012 13:29'!
calculateBezier: aPointList
	"comment stating purpose of message"

	| bernstein0 bernstein1 bernstein2 bernstein3 calculatedPoint step stepsize t|
	stepsize := 10.
	(aPointList size = 4) ifTrue: [
		step := 1 / (stepsize - 1).
		1 to: stepsize do: [ :i |
			t := i * step.
			bernstein0 := (1 - t) * (1 - t) * (1 - t).
			bernstein1 := 3 * t * (1 - t) * (1 - t).
			bernstein2 := 3 * t * t * (1 - t).
			bernstein3 := t * t * t.
			calculatedPoint := (aPointList at: 1) * bernstein0.
			calculatedPoint := calculatedPoint + ((aPointList at: 1) * bernstein1).
			calculatedPoint := calculatedPoint + ((aPointList at: 2) * bernstein2).
			calculatedPoint := calculatedPoint + ((aPointList at: 3) * bernstein3).
			"self halt."
			self addTesselatedPoint: calculatedPoint.
		]].! !

!BSpline methodsFor: 'rendering' stamp: 'CR 10/21/2012 17:37'!
drawALineFrom: aPoint To: anotherPoint
	"drawing a circle."
	| morph |
	morph := LineMorph from: aPoint to: anotherPoint color: Color black width: 2.
	morph openInWorld.! !

!BSpline methodsFor: 'initialize' stamp: 'CR 10/23/2012 14:05'!
initialize: aPointList
	"calculates the tesselatedPoints from the given control points"

	tesselatedPoints := OrderedCollection new.
	self calculateBSpline: aPointList.
! !

!BSpline methodsFor: 'rendering' stamp: 'CR 10/23/2012 14:11'!
render

	ShapeRenderer drawPolyLine: self tesselatedPoints.! !

!BSpline methodsFor: 'accessing' stamp: 'CR 10/21/2012 17:27'!
tesselatedPoints
	"Answer the value of tesselatedPoints"

	^ tesselatedPoints! !

!BSpline methodsFor: 'accessing' stamp: 'CR 10/21/2012 17:27'!
tesselatedPoints: anObject
	"Set the value of tesselatedPoints"

	tesselatedPoints := anObject! !

Object subclass: #FirstClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bundleview'!

!FirstClass methodsFor: 'as yet unclassified' stamp: 'SL 10/18/2012 17:03'!
countBackwards: x
	"counts a given number backwards till zero"
	| temp |
	temp := x.
	[temp > 0] whileTrue: [
		temp := temp - 1.
		Transcript show: temp
	].! !

!FirstClass methodsFor: 'as yet unclassified' stamp: 'SL 10/19/2012 14:00'!
drawACircleAt: aPoint
	"drawing a circle."
	| morph size |
	size := 20.
	morph := CircleMorph new.
	morph bounds: (Rectangle origin: (aPoint - ((size@size)/2)) extent: (size@size)).
	morph color: Color random.
	"morph setVertices: (OrderedCollection with: 100@100 with: 200@100 with: 100@200).
	morph color: (Color h: 40 s: 0.7 v: 0.6 alpha: 0.5).
	morph borderWidth: 5.
	morph borderColor:  (Color h: 170 s: 0.7 v: 0.6 alpha: 0.6).
	morph makeOpen."
	morph openInWorld.! !

!FirstClass methodsFor: 'as yet unclassified' stamp: 'SL 10/19/2012 14:24'!
drawALineFrom: aPoint To: anotherPoint
	"drawing a circle."
	| morph |
	morph := LineMorph from: aPoint to: anotherPoint color: Color black width: 2.
	morph openInWorld.! !

!FirstClass methodsFor: 'as yet unclassified' stamp: 'SL 10/19/2012 13:30'!
drawAPolyLine
	"showcase for drawing a polygon line."
	| morph |
	morph := PolygonMorph new.
	morph setVertices: (OrderedCollection with: 100@100 with: 200@100 with: 100@200).
	morph color: (Color h: 40 s: 0.7 v: 0.6 alpha: 0.5).
	morph borderWidth: 5.
	morph borderColor:  (Color h: 170 s: 0.7 v: 0.6 alpha: 0.6).
	morph makeOpen.
	morph openInWorld.! !

!FirstClass methodsFor: 'as yet unclassified' stamp: 'SL 10/17/2012 15:57'!
firstMethod
	^self.! !

Object subclass: #Node
	instanceVariableNames: 'children parent relations weight x y size representation lineRepresentations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bundleview'!

!Node commentStamp: 'CR 10/19/2012 10:33' prior: 0!
A Node is a part of a graph data structure. It knows its parent and its children as well of all relations that connect the node to an other node.

Instance Variables
	children:		<Object>
	parent:		<Object>
	relations:		<Object>

children
	- xxxxx

parent
	- xxxxx

relations
	- xxxxx
!

!Node methodsFor: 'accessing' stamp: 'SL 10/19/2012 15:11'!
addChild: aChild
	children add: aChild.
	aChild parent: self.! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 17:37'!
addLineRepresentation: aLine
	lineRepresentations add: aLine.! !

!Node methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:29'!
addRelation: aRelation
	relations add: aRelation.! !

!Node methodsFor: 'as yet unclassified' stamp: 'SL 10/22/2012 15:40'!
calculateNumberOfLeafs
	"calculate the number of leaf nodes in this (sub-)tree."
	| numberOfLeafs |
	numberOfLeafs := 0.
	self children
		ifEmpty: [numberOfLeafs := 1]
		ifNotEmpty: [self children do: [ :child | numberOfLeafs := numberOfLeafs + child calculateNumberOfLeafs]].
	^numberOfLeafs! !

!Node methodsFor: 'as yet unclassified' stamp: 'SL 10/23/2012 09:57'!
calculateRadialTreeLayoutAtPoint: rootPoint nodeSize: nodeSize nodePadding: nodePadding layerDistance: layerDistance
	"."
	self 
		calculateRadialTreeLayoutAtPoint: rootPoint 
		startAngle: 0 
		endAngle: 2 * 3.1415
		nodeSize: nodeSize 
		nodePadding: nodePadding 
		layerDistance: layerDistance 
		totalNumberOfLeafs: (self calculateNumberOfLeafs).
	^self.! !

!Node methodsFor: 'as yet unclassified' stamp: 'SL 10/23/2012 10:01'!
calculateRadialTreeLayoutAtPoint: rootPoint startAngle: startAngle endAngle: endAngle nodeSize: nodeSize nodePadding: nodePadding layerDistance: layerDistance totalNumberOfLeafs: totalNumberOfLeafs
	"."
	| abc angle anglePerLeaf childStartAngle childEndAngle |
	angle := (startAngle + endAngle) / 2.
	self x: angle sin * (self depth - 1) * layerDistance + rootPoint x.
	self y: angle cos * (self depth - 1) * layerDistance + rootPoint y.
	self size: nodeSize.
	
	anglePerLeaf := (endAngle - startAngle) / totalNumberOfLeafs.
	childStartAngle := startAngle.
	self children do: [ :child | 
		abc := 0.
		childEndAngle := childStartAngle + (anglePerLeaf * (child calculateNumberOfLeafs)).
		child 
			calculateRadialTreeLayoutAtPoint: rootPoint 
			startAngle: childStartAngle
			endAngle: childEndAngle
			nodeSize: nodeSize
			nodePadding: nodePadding 
			layerDistance: layerDistance 
			totalNumberOfLeafs: (child calculateNumberOfLeafs).
		childStartAngle := childEndAngle.
	].
	^self.! !

!Node methodsFor: 'as yet unclassified' stamp: 'SL 10/22/2012 15:46'!
calculateRootedTreeLayoutAtPoint: rootPoint nodeSize: nodeSize nodePadding: nodePadding layerDistance: layerDistance
	"."
	| childLeft length childX childY |
	self x: rootPoint x.
	self y: rootPoint y.
	self size: nodeSize.

	length := (nodeSize + nodePadding) * self calculateNumberOfLeafs.
	childLeft := rootPoint x - (length / 2).
	
	childX := childLeft.
	childY := rootPoint y + layerDistance.
	
	self children do: [ :child | 
		childX := childX + ((nodeSize + nodePadding) * child calculateNumberOfLeafs /2).
		child calculateRootedTreeLayoutAtPoint: (childX@childY) 
			nodeSize: nodeSize 
			nodePadding: nodePadding 
			layerDistance: layerDistance.
		childX := childX + ((nodeSize + nodePadding) * child calculateNumberOfLeafs /2).
	].
	^self.! !

!Node methodsFor: 'as yet unclassified' stamp: 'SL 10/21/2012 16:12'!
calculateWeight
	"calculate the weight of this node depending on the weight of the children"

	| calculatedWeight |
	calculatedWeight := 0.
	self children
		ifEmpty: [calculatedWeight := self weight]
		ifNotEmpty: [self children do: [ :child | calculatedWeight := calculatedWeight + child calculateWeight]].
	self weight: calculatedWeight.
	^calculatedWeight! !

!Node methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:24'!
children
	^children! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
children: anObject
	"Set the value of children"

	children := anObject! !

!Node methodsFor: 'accessing' stamp: 'SL 10/19/2012 15:37'!
depth
	self isRoot
		ifTrue: [^1]
		ifFalse: [^1 + parent depth].! !

!Node methodsFor: 'as yet unclassified' stamp: 'SL 10/23/2012 10:07'!
drawWithLines
	"."
	self children do: [ :child |
		lineRepresentations add: (ShapeRenderer drawLineFrom: (self x)@(self y) to: (child x)@(child y)).
		child drawWithLines.
	].	
	self representation: (ShapeRenderer drawCirlceAt: ((self x)@(self y)) size: size).
	^self.! !

!Node methodsFor: 'accessing' stamp: 'SL 10/19/2012 15:59'!
getMaxDepth
	"get depth of the whole (sub-)tree."
	| maxDepthOfChildren |
	maxDepthOfChildren := children inject: 0 into: [:prev :child | (prev > child getMaxDepth)
		ifTrue: [prev]
		ifFalse: [child getMaxDepth]].
	^ maxDepthOfChildren + 1.
! !

!Node methodsFor: 'initializing' stamp: 'SL 10/21/2012 17:37'!
initialize
	super initialize.
	weight := 1.
	children := OrderedCollection new.
	relations := OrderedCollection new.
	lineRepresentations := OrderedCollection new.! !

!Node methodsFor: 'accessing' stamp: 'SL 10/19/2012 15:26'!
isLeaf
	^ children isEmpty.! !

!Node methodsFor: 'accessing' stamp: 'SL 10/19/2012 15:07'!
isRoot
	^ parent isNil.! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 17:36'!
lineRepresentations
	"Answer the value of lineRepresentations"

	^ lineRepresentations! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 17:36'!
lineRepresentations: anObject
	"Set the value of lineRepresentations"

	lineRepresentations := anObject! !

!Node methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:13'!
parent
	"Answer the value of parent"

	^ parent! !

!Node methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:14'!
parent: anObject
	"Set the value of parent"

	parent := anObject! !

!Node methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:30'!
relations
	^relations! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
relations: anObject
	"Set the value of relations"

	relations := anObject! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
representation
	"Answer the value of representation"

	^ representation! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
representation: anObject
	"Set the value of representation"

	representation := anObject! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 17:28'!
size
	^size! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 17:27'!
size: anObject
	"Set the value of size"

	size := anObject! !

!Node methodsFor: 'accessing' stamp: 'CR 10/19/2012 14:23'!
weight
	^weight! !

!Node methodsFor: 'accessing' stamp: 'CR 10/19/2012 14:23'!
weight: aNumber
	weight := aNumber.! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
x
	"Answer the value of x"

	^ x! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
x: anObject
	"Set the value of x"

	x := anObject! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
y
	"Answer the value of y"

	^ y! !

!Node methodsFor: 'accessing' stamp: 'SL 10/21/2012 16:30'!
y: anObject
	"Set the value of y"

	y := anObject! !

Object subclass: #Relation
	instanceVariableNames: 'source destination'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bundleview'!

!Relation commentStamp: 'CR 10/19/2012 10:34' prior: 0!
A Relation describes an usally non hierarchical connection between two nodes.

Instance Variables
	destination:		<Object>
	source:		<Object>

destination
	- xxxxx

source
	- xxxxx
!

!Relation methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:28'!
destination
	"Answer the value of destination"

	^ destination! !

!Relation methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:28'!
destination: anObject
	"Set the value of destination"

	destination := anObject! !

!Relation methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:28'!
source
	"Answer the value of source"

	^ source! !

!Relation methodsFor: 'accessing' stamp: 'CR 10/19/2012 10:28'!
source: anObject
	"Set the value of source"

	source := anObject! !

Object subclass: #RootedTreeRenderer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bundleview'!

!RootedTreeRenderer class methodsFor: 'as yet unclassified' stamp: 'SL 10/21/2012 15:05'!
render: aNode
	"renders a tree"
	self render: aNode.
	^self! !

!RootedTreeRenderer class methodsFor: 'as yet unclassified' stamp: 'SL 10/21/2012 15:05'!
render: aNode At: aPoint
	^self! !

Object subclass: #ShapeRenderer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bundleview'!

!ShapeRenderer class methodsFor: 'as yet unclassified' stamp: 'SL 10/21/2012 18:19'!
drawCirlceAt: middlePoint size: size
	"draws a simple cirlce"
	| circle |
	circle := CircleMorph new.
	circle bounds: (Rectangle origin: (middlePoint - ((size@size)/2)) extent: (size@size)).
	circle color: Color random.
	circle openInWorld.
	^circle.! !

!ShapeRenderer class methodsFor: 'as yet unclassified' stamp: 'SL 10/21/2012 18:14'!
drawLineFrom: sourcePoint to: destinationPoint
	"draws a simple line"
	^(LineMorph
		from: (sourcePoint x)@(sourcePoint y)
		to: (destinationPoint x)@(destinationPoint y)
		color: Color black
		width: 2)
	openInWorld.! !

!ShapeRenderer class methodsFor: 'as yet unclassified' stamp: 'CR 10/23/2012 14:11'!
drawPolyLine: aPointList

	| morph |
	morph := PolygonMorph new.
	morph setVertices: aPointList.
	morph borderWidth: 2.
	morph makeOpen.
	morph openInWorld.! !

Object subclass: #TreeBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bundleview'!

!TreeBuilder class methodsFor: 'as yet unclassified' stamp: 'SL 10/22/2012 18:05'!
buildLargeTree
	"creates a simple Treestructure"
	|			 				node1 
				node11 			       			node12 
	node111    node112    node113 		node121 		node122 
										      			node1221	node1222 |
	node1 := Node new.
	node11 := Node new.
	node111 := Node new.
	node112 := Node new.
	node113 := Node new.
	node12 := Node new.
	node121 := Node new.
	node122 := Node new.
	node1221 := Node new.
	node1222 := Node new.
	
	node1 addChild: node11.
	node11 addChild: node111.
	node11 addChild: node112.
	node11 addChild: node113.
	node11 addChild: Node new.
	node111 addChild: Node new.
	node111 addChild: Node new.
	node112 addChild: Node new.
	node112 addChild: Node new.
	node113 addChild: Node new.
	node113 addChild: Node new.
	node113 addChild: Node new.
	node1 addChild: node12.
	node12 addChild: node121.
	node12 addChild: node122.
	node122 addChild: node1221.
	node1221 addChild: Node new.
	node1221 addChild: Node new.
	node1221 addChild: Node new.
	node122 addChild: node1222.
	node1222 addChild: Node new.
	node1222 addChild: Node new.
	node1222 addChild: Node new.
	node1222 addChild: Node new.
	node1222 addChild: Node new.
	node1222 addChild: Node new.
	node1222 addChild: Node new.
	^node1.! !

!TreeBuilder class methodsFor: 'as yet unclassified' stamp: 'SL 10/23/2012 10:06'!
buildTestTree
	"creates a simple Treestructure"
	|			 				node1 
				node11 			       			node12 
	node111       node112 		node113 		node121 		node122
										      			node1221	node1222 |
	node1 := Node new.
	node11 := Node new.
	node111 := Node new.
	node112 := Node new.
	node113 := Node new.
	node12 := Node new.
	node121 := Node new.
	node122 := Node new.
	node1221 := Node new.
	node1222 := Node new.
	
	node1 addChild: node11.
	node11 addChild: node111.
	node11 addChild: node112.
	node11 addChild: node113.
	node1 addChild: node12.
	node12 addChild: node121.
	node12 addChild: node122.
	node122 addChild: node1221.
	node122 addChild: node1222.
	^node1.! !
